import string
import six
import logging

from ipalib import api, errors, output, _
from ipalib import Str, StrEnum, Bool, Int, _
from ipalib.plugable import Registry
from . import baseuser
from .baseuser import (
    baseuser,
    baseuser_add,
    baseuser_del,
    baseuser_mod,
    baseuser_find,
    baseuser_show,
    NO_UPG_MAGIC,
    UPG_DEFINITION_DN,
    baseuser_output_params,
    validate_nsaccountlock,
    convert_nsaccountlock,
    fix_addressbook_permission_bindrule,
    baseuser_add_manager,
    baseuser_remove_manager,
    baseuser_add_principal,
    baseuser_remove_principal)
from . import baseldap
from .baseldap import (LDAPObject, LDAPCreate, LDAPDelete,
                                     LDAPUpdate, LDAPSearch, LDAPRetrieve,
                                     LDAPQuery, LDAPAddMember, LDAPRemoveMember,
                                     add_external_pre_callback,
                                     add_external_post_callback,
                                     remove_external_post_callback,
                                     output, entry_to_dict, pkey_to_value,
                                     LDAPAddAttribute, LDAPRemoveAttribute,
                                     LDAPModAttribute)
from .hbacrule import is_all
from ipalib import _, ngettext
from ipalib.util import validate_hostmask
from ipapython.dn import DN

if six.PY3:
    unicode = str


__doc__ = _("""
Parsec policy
""")

register = Registry()

topic = 'parsec'

@register()
class maclevels(baseldap.LDAPObject):
    """
    MAC Level
    """
    container_dn = DN(('cn', 'mac-levels'),
                      ('cn', 'mac-services'),
                      ('cn', 'parsec-configs'),
                      ('cn', 'etc'))
    object_name = _('maclevel')
    object_name_plural = _('maclevels')
    object_class = ['top', 'nscontainer', 'x-ald-mac-level-object']
    default_attributes = ['cn', 'x-ald-mac-level']
    search_attributes = ['cn', 'x-ald-mac-level']


    label = _('MAC Levels')
    label_singular = _('MAC Level')

    takes_params = (
        Str('cn',
            cli_name='maclevel_name',
            label=_('MAC Level'),
            primary_key=True,
        ),
        Int('x-ald-mac-level?',
            cli_name='maclevel',
            label=_('Levels of the MAC'),
            doc=_('integer to order the MAC leveles'),
            minvalue=0,
            maxvalue=255,
        ),
    )
    allow_rename = True


@register()
class maclevels_add(LDAPCreate):
    __doc__ = _('Create new MAC Level.')

    msg_summary = _('Added MAC Level"%(value)s"')

@register()
class maclevels_del(LDAPDelete):
    __doc__ = _('Delete MAC Level.')

    msg_summary = _('Deleted MAC Level"%(value)s"')

    def pre_callback(self, ldap, dn, *keys, **options):
        assert isinstance(dn, DN)
        logging.debug('PARSEC: maclevels_del Enters keys=%s', keys)
        str = ''
        for key in keys:
            logging.debug('PARSEC: maclevels_del key=%s', key)
            try:
                lev = ldap.get_entry( DN(('cn', key), ('cn', 'mac-levels'), ('cn', 'mac-services'), ('cn', 'parsec-configs'), ('cn', 'etc'), api.env.basedn), attrs_list=['x-ald-mac-level'])
            except:
                logging.debug('PARSEC: maclevels_del can not find maclevel %s value', key)
                return None
            try:
                lev = int( lev['x-ald-mac-level'][0], 10)
            except:
                logging.debug('PARSEC: maclevels_del maclevel %s has no value - OK to delete', key)
                break
            logging.debug('PARSEC: maclevels_del lev=%s', lev)
            try:
                (users, truncated) = ldap.find_entries(
                '(&(xaldusermacmax=*)(xaldusermacmin=*))', ['uid','xaldusermacmax','xaldusermacmin'],
                DN(('cn', 'users'), ('cn', 'accounts'), api.env.basedn),
                ldap.SCOPE_ONELEVEL
                )
                for user in users:
                    levmax = int(user['xaldusermacmax'][0],10)
                    levmin = int(user['xaldusermacmin'][0],10)
                    if levmax >= lev >= levmin:
                        if len(str) > 0:
                            str = str + ', '
                        str = str + '"' + user['uid'][0] + '"';
                logging.debug('PARSEC: maclevels_del users=%s', str)
            except errors.NotFound:
                logging.debug('PARSEC: maclevels_del %s has no dependent users', key)
            try:
                (devices, truncated) = ldap.find_entries(
                '(x-ald-mac-level=%s)' % (lev), ['cn'],
                DN(('cn', 'devices'), ('cn', 'dev-services'), ('cn', 'parsec-configs'), ('cn', 'etc'), api.env.basedn),
                ldap.SCOPE_ONELEVEL
                )
                for dev in devices:
                    if len(str) > 0:
                        str = str + ', '
                    str = str + '"' + dev['cn'][0] + '"';
                logging.debug('PARSEC: maclevels_del devices=%s', str)
            except errors.NotFound:
                logging.debug('PARSEC: maclevels_del %s has no dependent devices', key)
            if len(str) > 0:
                raise errors.DependentEntry(label=u'users or devices', key=key, dependent=str)
                return None
        return dn

@register()
class maclevels_mod(LDAPUpdate):

    __doc__ = _('Modify MAC Level.')

    msg_summary = _('Modified MAC Level"%(value)s"')




@register()
class maclevels_find(LDAPSearch):
    __doc__ = _('Search for MAC Level.')

    msg_summary = ngettext(
        '%(count)d MAC Level matched',
        '%(count)d MAC Level matched', 0
    )

@register()
class maclevels_show(LDAPRetrieve):
    __doc__ = _('Display MAC Level.')

"""
@register()
class maclevels_add_member(LDAPAddMember):
    __doc__ = _('Add members to MAC Group.')

@register()
class maclevels_remove_member(LDAPRemoveMember):
    __doc__ = _('Remove members from MAC Group.')

"""

@register()
class miclevels(baseldap.LDAPObject):
    """
    MIC Levels
    """
    container_dn = DN(('cn', 'mic-levels'),
                      ('cn', 'mic-services'),
                      ('cn', 'parsec-configs'),
                      ('cn', 'etc'))
    object_name = _('miclevel')
    object_name_plural = _('miclevels')
    object_class = ['top', 'nscontainer', 'x-ald-user']
    default_attributes = ['cn', 'x-ald-user-mic-level']
    search_attributes = ['cn', 'x-ald-user-mic-level']

    label = _('MIC Levels')
    label_singular = _('MIC Level')

    takes_params = (
        Str('cn',
            cli_name='miclevel_name',
            label=_('MIC Level'),
            primary_key=True,
        ),
        Int('x-ald-user-mic-level',
            cli_name='miclevel',
            label=_('Levels of the MIC'),
            doc=_('integer to order the MIC leveles'),
            minvalue=0,
            maxvalue=255,
        ),
    )
    allow_rename = True


#    def get_dn(self, *keys, **kwargs):
#        if keys[-1] is not None:
#            return self.api.Object.user.get_dn(*keys, **kwargs)
#        return DN(self.container_dn, api.env.basedn)

@register()
class miclevels_add(LDAPCreate):
    __doc__ = _('Create new MIC level.')

#    def pre_callback(self, ldap, dn, entry_attrs, attrs_list, *keys, **options):
#        assert isinstance(dn, DN)
#        self.obj.check_order_uniqueness(*keys, **options)
#        return dn

    msg_summary = _('Added MIC Level "%(value)s"')

@register()
class miclevels_del(LDAPDelete):
    __doc__ = _('Delete MIC Level.')

    msg_summary = _('Deleted MIC Level "%(value)s"')

    def pre_callback(self, ldap, dn, *keys, **options):
        assert isinstance(dn, DN)
        logging.debug('PARSEC: miclevels_del Enters keys=%s', keys)
        str = ''
        for key in keys:
            logging.debug('PARSEC: miclevels_del key=%s', key)
            try:
                lev = ldap.get_entry( DN(('cn', key), ('cn', 'mic-levels'), ('cn', 'mic-services'), ('cn', 'parsec-configs'), ('cn', 'etc'), api.env.basedn), attrs_list=['x-ald-user-mic-level'])
                lev=lev['x-ald-user-mic-level'][0]
            except:
                logging.debug('PARSEC: miclevels_del can not find miclevel %s value', key)
                return None
            logging.debug('PARSEC: miclevels_del lev=%s', lev)

            mask=0
            try:
                (levels, truncated) = ldap.find_entries(
                '(!(cn=%s))' % (key), ['x-ald-user-mic-level'],
                DN( ('cn', 'mic-levels'), ('cn', 'mic-services'), ('cn', 'parsec-configs'), ('cn', 'etc'), api.env.basedn),
                ldap.SCOPE_ONELEVEL
                )
                logging.debug('PARSEC: miclevels_del got levels %s', levels)
                for level in levels:
                    logging.debug('PARSEC: miclevels_del level %s', level)
                    mask |= int(level['x-ald-user-mic-level'][0])
            except errors.NotFound:
                logging.debug('PARSEC: miclevels_del no more levels found')
            mask = ~mask
            mask &= int(lev)
            logging.debug('PARSEC: miclevels_del mask=%s', mask)
            if mask:
                try:
                    (users, truncated) = ldap.find_entries(
                    '(&(x-ald-user-mic-level=*)(!(x-ald-user-mic-level=0)))', ['uid', 'x-ald-user-mic-level'],
                    DN(('cn', 'users'), ('cn', 'accounts'), api.env.basedn),
                    ldap.SCOPE_ONELEVEL
                    )
                    logging.debug('PARSEC: miclevels_del users=%s', str)
                    for user in users:
                        logging.debug('PARSEC: miclevels_del user=%s', user)
                        if mask & int(user['x-ald-user-mic-level'][0]):
                            if len(str) > 0:
                                str = str + ', '
                            str = str + '"' + user['uid'][0] + '"';
                except errors.NotFound:
                    logging.debug('PARSEC: miclevels_del %s has no dependent users', key)
                if len(str) > 0:
                    raise errors.DependentEntry(label=u'users', key=key, dependent=str)
                    return None
        return dn

@register()
class miclevels_mod(LDAPUpdate):
    __doc__ = _('Modify MIC Level.')

    msg_summary = _('Modified MIC Level"%(value)s"')

@register()
class miclevels_find(LDAPSearch):
    __doc__ = _('Search for MIC Level.')

    msg_summary = ngettext(
        '%(count)d MIC Level matched',
        '%(count)d MIC Level matched', 0
    )

@register()
class miclevels_show(LDAPRetrieve):
    __doc__ = _('Display MIC Level.')


