import string
import six
import logging

from ipalib import api, errors, output, _
from ipalib import Str, StrEnum, Bool, Int, _
from ipalib.plugable import Registry
from . import baseldap
from .baseldap import (LDAPObject, LDAPCreate, LDAPDelete,
                                     LDAPUpdate, LDAPSearch, LDAPRetrieve,
                                     LDAPQuery, LDAPAddMember, LDAPRemoveMember,
                                     add_external_pre_callback,
                                     add_external_post_callback,
                                     remove_external_post_callback,
                                     output, entry_to_dict, pkey_to_value,
                                     LDAPAddAttribute, LDAPRemoveAttribute,
                                     LDAPModAttribute)
from .hbacrule import is_all
from ipalib import _, ngettext
from ipalib.util import validate_hostmask
from ipapython.dn import DN

if six.PY3:
    unicode = str


__doc__ = _("""
Parsec registered devices rules
""")

register = Registry()

topic = 'parsecrule'

@register()
class parsecrule(baseldap.LDAPObject):
    container_dn = DN(('cn', 'rules'),
                      ('cn', 'dev-services'),
                      ('cn', 'parsec-configs'),
                      ('cn', 'etc'))
    object_name = _('parsecrule')
    object_name_plural = _('parsecrules')
    object_class = ['top', 'nscontainer', 'nestedgroup', 'x-ald-device-rule']
    default_attributes = ['cn', 'description', 'x-ald-devac-attr', 'x-ald-devac-status', 'member']
    search_attributes = ['cn', 'description', 'x-ald-devac-attr', 'x-ald-devac-status', 'member']

    attribute_members = {
        'member': ['parsecdevice'],
    }

    label = _('Devices Rules')
    label_singular = _('Device Rule')

    takes_params = (
        Str('cn',
            cli_name='rule_name',
            label=_('Name of rule'),
            primary_key=True,
        ),
        Str('description?',
            cli_name='rule_description',
            label=_('Rule description'),
            primary_key=False,
        ),
        Bool('x-ald-devac-status?',
            cli_name='rule_status',
            label=_('Rule status'),
            primary_key=False,
        ),
        Str('x-ald-devac-attr',
            cli_name='rule_attr',
            label=_('Rule attr'),
            primary_key=False,
        ),
    )

@register()
class parsecrule_find(LDAPSearch):
    __doc__ = _('Search for registered devices.')

    msg_summary = ngettext(
        '%(count)d device rule matched',
        '%(count)d device rule matched', 0
    )

@register()
class parsecrule_show(LDAPRetrieve):
    __doc__ = _('Display device rule.')


@register()
class parsecrule_mod(LDAPUpdate):
    __doc__ = _('Modify device rule.')

    msg_summary = _('Modified device rule"%(value)s"')

@register()
class parsecrule_add(LDAPCreate):
    __doc__ = _('Create new rule.')

    msg_summary = _('Added rule "%(value)s"')

    def pre_callback(self, ldap, dn, entry_attrs, attrs_list, *keys, **options):

        if entry_attrs.get('x-ald-devac-status') is None:
            entry_attrs['x-ald-devac-status'] = False
        return dn


@register()
class parsecrule_del(LDAPDelete):
    __doc__ = _('Delete device rule.')

    msg_summary = _('Deleted device rule"%(value)s"')

    def pre_callback(self, ldap, dn, *keys, **options):
        assert isinstance(dn, DN)
        str = ''
        for rulename in keys:
            logging.debug('PARSEC: parsecrule_del rulename=%s', rulename)
            try:
                (devices, truncated) = ldap.find_entries(
                '(x-ald-devac-rule=%s)' % (rulename), ['cn'],
                DN(('cn', 'devices'), ('cn', 'dev-services'), ('cn', 'parsec-configs'), ('cn', 'etc'), api.env.basedn),
                ldap.SCOPE_ONELEVEL
                )

                for dev in devices:
                    if len(str) > 0:
                        str = str + ', '
                    str = str + '"' + dev['cn'][0] + '"';
                logging.debug('PARSEC: parsecrule_del devices=%s', str)
            except errors.NotFound:
                logging.debug('PARSEC: parsecrule_del OK to delete')
            if len(str) > 0:
                raise errors.DependentEntry(label=u'registered device', key=rulename, dependent=str)
                return None
        return dn

def parsecrules_update(self, mode, ldap, completed, failed, dn, entry_attrs, *keys, **options):
    devicenames = keys[-1]['parsecdevice']
    for devicename in devicenames:
        memberdevice = DN(('cn', devicename), ('cn', 'devices'), ('cn', 'dev-services'), ('cn', 'parsec-configs'), ('cn', 'etc'), api.env.basedn)
        rules_new = []
        try:
            (rules, truncated) = ldap.find_entries(
            'member=%s' % (memberdevice), ['cn'],
            DN(('cn', 'rules'), ('cn', 'dev-services'), ('cn', 'parsec-configs'), ('cn', 'etc'), api.env.basedn),
            ldap.SCOPE_ONELEVEL)
            for rule in rules:
                logging.debug('PARSEC: parsecrules_update rule: %s', rule['cn'][0])
                rules_new = rules_new + rule['cn']
        except errors.NotFound:
            logging.debug('PARSEC: parsecrules_update no rules found')

        logging.debug('PARSEC: parsecrules_update new rules: %s', rules_new)
        entry = ldap.get_entry( memberdevice, attrs_list=['x-ald-devac-rule'])
        entry['x-ald-devac-rule'] = rules_new
        ldap.update_entry(entry)
    return dn

@register()
class parsecrule_add_parsecrule(LDAPAddMember):

    __doc__ = _('Add registered device rule.')

    member_attributes = ['member']
    has_output_params = LDAPAddMember.has_output_params

    def post_callback(self, ldap, completed, failed, dn, entry_attrs, *keys, **options):
        assert isinstance(dn, DN)

        parsecrules_update(self, 'add', ldap, completed, failed, dn, entry_attrs, keys, options)

        return (completed, dn)

@register()
class parsecrule_remove_parsecrule(LDAPRemoveMember):
    __doc__ = _('Remove registered device rule.')

    member_attributes = ['member']
    has_output_params = LDAPRemoveMember.has_output_params

    def post_callback(self, ldap, completed, failed, dn, entry_attrs, *keys, **options):
        assert isinstance(dn, DN)

        parsecrules_update(self, 'del', ldap, completed, failed, dn, entry_attrs, keys, options)

        return (completed, dn)
